package de.intevation.flys.artifacts.model;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;
import de.intevation.artifacts.DataProvider;

import de.intevation.artifactdatabase.state.DefaultFacet;

import de.intevation.flys.artifacts.MainValuesArtifact;
import de.intevation.flys.artifacts.math.Linear;
import de.intevation.flys.jfree.FLYSAnnotation;
import de.intevation.flys.jfree.StickyAxisAnnotation;

import de.intevation.flys.exports.DurationCurveGenerator;


/**
 * Facet to show Main Q Values.
 * TODO Join with W implementation.
 */
public class MainValuesQFacet
extends      DefaultFacet
implements   FacetTypes {

    /** Own logger. */
    private static Logger logger = Logger.getLogger(RelativePointFacet.class);

    /** Do we want MainValues at Gauge (not interpolated)? */
    protected boolean isAtGauge;

    /** Trivial Constructor. */
    public MainValuesQFacet(String name, String description, boolean atGauge) {
        this.description = description;
        this.name        = name;
        this.index       = 0;
        this.isAtGauge   = atGauge;
    }

    /**
     * Set the hit-point in Q where a line drawn from the axis would hit the
     * curve in WQDay (if hit).
     * Employ linear interpolation.
     */
    protected static void setHitPoint(WQDay wqday, StickyAxisAnnotation annotation) {
        int idx = 0;
        float q = annotation.getPos();
        boolean qIncreases = wqday.getQ(0) < wqday.getQ(wqday.size()-1);
        if (qIncreases) {
            while (idx < wqday.size() && wqday.getQ(idx) < q) {
                idx++;
            }
        }
        else {
            idx = wqday.size() -1;
            while (idx > 0 && wqday.getQ(idx) > q) {
                idx--;
            }
        }

        double day = 0d;
        int mod = (qIncreases) ? -1 : +1;
        if (idx != 0 && idx <= wqday.size()-1) {
            day = Linear.linear(q, wqday.getQ(idx +mod), wqday.getQ(idx),
                wqday.getDay(idx+mod), wqday.getDay(idx));
            annotation.setHitPoint((float)day);
        }
        else {
            logger.debug("StickyAnnotation does not hit wqday curve");
        }
    }


    /**
     * Returns the data this facet requires.
     *
     * @param artifact the owner artifact.
     * @param context  the CallContext (ignored).
     *
     * @return the data.
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        MainValuesArtifact mvArtifact = (MainValuesArtifact) artifact;

        List<NamedDouble>          qs = mvArtifact.getMainValuesQ(isAtGauge);
        List<StickyAxisAnnotation> xy = new ArrayList<StickyAxisAnnotation>();

        WQDay wqdays = null;
        List<DataProvider> providers = context.
            getDataProvider(DurationCurveFacet.BB_DURATIONCURVE);
        if (providers.size() < 1) {
            logger.warn("Could not find durationcurve data provider.");
        }
        else {
            wqdays = (WQDay) providers.get(0).provideData(
                DurationCurveFacet.BB_DURATIONCURVE,
                null,
                context);
        }

        // Rather specific case, Q-Annotations at a maybe second yaxis.
        StickyAxisAnnotation annotation = null;
        if (this.name.equals(DURATION_MAINVALUES_Q)) {
            for (NamedDouble q: qs) {
                annotation =
                    new StickyAxisAnnotation(
                        q.getName(),
                        (float) q.getValue(),
                        StickyAxisAnnotation.SimpleAxis.Y_AXIS,
                        DurationCurveGenerator.YAXIS.Q.idx);
                xy.add(annotation);
                if (wqdays != null) {
                    setHitPoint(wqdays, annotation);
                }
            }
        }
        else {
            for (NamedDouble q: qs) {
                annotation =
                    new StickyAxisAnnotation(
                        q.getName(),
                        (float) q.getValue(),
                        StickyAxisAnnotation.SimpleAxis.X_AXIS);
                xy.add(annotation);
                if (wqdays != null) {
                    setHitPoint(wqdays, annotation);
                }
            }
        }

        return new FLYSAnnotation(description, xy);
    }


    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public MainValuesQFacet deepCopy() {
        MainValuesQFacet copy = new MainValuesQFacet(this.name,
            description, this.isAtGauge);
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
